@@ -13,10 +13,8 @@ from ..utils import register_linting_configuration_options
 CONFIG_NAME = "django_migration_linter"
 DEFAULT_CONFIG_FILES = (
     ".{}.cfg".format(CONFIG_NAME),
-    "pyproject.toml",
     "setup.cfg",
     "tox.ini",
-    ".editorconfig",
 )
 
 
@@ -130,7 +128,7 @@ class Command(BaseCommand):
             logging.basicConfig(format="%(message)s")
 
         config_parser = configparser.ConfigParser()
-        config_parser.read(DEFAULT_CONFIG_FILES)
+        config_parser.read(DEFAULT_CONFIG_FILES, encoding="utf-8")
 
         ignore_name_contains = options["ignore_name_contains"] or config_parser.get(
             CONFIG_NAME, "ignore_name_contains", fallback=None
