@@ -5,26 +5,26 @@ import inspect
 import logging
 import os
 import re
-from subprocess import Popen, PIPE
+from enum import Enum, unique
+from subprocess import PIPE, Popen
 
 from django.conf import settings
 from django.core.management import call_command
-from django.db import DEFAULT_DB_ALIAS, connections, ProgrammingError
-from django.db.migrations import RunPython
-from enum import Enum, unique
+from django.db import DEFAULT_DB_ALIAS, ProgrammingError, connections
+from django.db.migrations import RunPython, RunSQL
 from six import PY2
 
 from .cache import Cache
 from .constants import (
     DEFAULT_CACHE_PATH,
-    EXPECTED_DATA_MIGRATION_ARGS,
     DJANGO_APPS_WITH_MIGRATIONS,
+    EXPECTED_DATA_MIGRATION_ARGS,
 )
-from .utils import clean_bytes_to_str, get_migration_abspath, split_migration_path
 from .operations import IgnoreMigration
 from .sql_analyser import analyse_sql_statements
+from .utils import clean_bytes_to_str, get_migration_abspath, split_migration_path
 
-logger = logging.getLogger(__name__)
+logger = logging.getLogger("django_migration_linter")
 
 
 @unique
@@ -45,6 +45,8 @@ class MigrationLinter(object):
         path=None,
         ignore_name_contains=None,
         ignore_name=None,
+        include_name_contains=None,
+        include_name=None,
         include_apps=None,
         exclude_apps=None,
         database=DEFAULT_DB_ALIAS,
@@ -60,6 +62,8 @@ class MigrationLinter(object):
         self.django_path = path
         self.ignore_name_contains = ignore_name_contains
         self.ignore_name = ignore_name or tuple()
+        self.include_name_contains = include_name_contains
+        self.include_name = include_name or tuple()
         self.include_apps = include_apps
         self.exclude_apps = exclude_apps
         self.exclude_migration_tests = exclude_migration_tests or []
@@ -72,11 +76,7 @@ class MigrationLinter(object):
         self.warnings_as_errors = warnings_as_errors
 
         # Initialise counters
-        self.nb_valid = 0
-        self.nb_ignored = 0
-        self.nb_warnings = 0
-        self.nb_erroneous = 0
-        self.nb_total = 0
+        self.reset_counters()
 
         # Initialise cache. Read from old, write to new to prune old entries.
         if self.should_use_cache():
@@ -91,6 +91,13 @@ class MigrationLinter(object):
             connection=connections[self.database], load=True
         )
 
+    def reset_counters(self):
+        self.nb_valid = 0
+        self.nb_ignored = 0
+        self.nb_warnings = 0
+        self.nb_erroneous = 0
+        self.nb_total = 0
+
     def should_use_cache(self):
         return self.django_path and not self.no_cache
 
@@ -289,8 +296,13 @@ class MigrationLinter(object):
 
     @classmethod
     def read_migrations_list(cls, migrations_file_path):
+        """
+        Returning an empty list is different from returning None here.
+        None: no file was specified and we should consider all migrations
+        Empty list: no migration found in the file and we should consider no migration
+        """
         if not migrations_file_path:
-            return []
+            return None
 
         migrations = []
         try:
@@ -300,7 +312,14 @@ class MigrationLinter(object):
                         app_label, name = split_migration_path(line)
                         migrations.append((app_label, name))
         except IOError:
-            logger.warning("Migrations list path not found %s", migrations_file_path)
+            logger.exception("Migrations list path not found %s", migrations_file_path)
+            raise Exception("Error while reading migrations list file")
+
+        if not migrations:
+            logger.info(
+                "No valid migration paths found in the migrations file %s",
+                migrations_file_path,
+            )
         return migrations
 
     def _gather_migrations_git(self, git_commit_id, migrations_list=None):
@@ -315,9 +334,19 @@ class MigrationLinter(object):
             # Only gather lines that include added migrations
             if self.is_migration_file(line):
                 app_label, name = split_migration_path(line)
-                if not migrations_list or (app_label, name) in migrations_list:
-                    migration = self.migration_loader.disk_migrations[app_label, name]
-                    migrations.append(migration)
+                if migrations_list is None or (app_label, name) in migrations_list:
+                    if (app_label, name) in self.migration_loader.disk_migrations:
+                        migration = self.migration_loader.disk_migrations[
+                            app_label, name
+                        ]
+                        migrations.append(migration)
+                    else:
+                        logger.info(
+                            "Found migration file (%s, %s) "
+                            "that is not present in loaded migration.",
+                            app_label,
+                            name,
+                        )
         diff_process.wait()
 
         if diff_process.returncode != 0:
@@ -334,7 +363,7 @@ class MigrationLinter(object):
             migration,
         ) in self.migration_loader.disk_migrations.items():
             if app_label not in DJANGO_APPS_WITH_MIGRATIONS:  # Prune Django apps
-                if not migrations_list or (app_label, name) in migrations_list:
+                if migrations_list is None or (app_label, name) in migrations_list:
                     yield migration
 
     def should_ignore_migration(self, app_label, migration_name, operations=()):
@@ -346,7 +375,12 @@ class MigrationLinter(object):
                 self.ignore_name_contains
                 and self.ignore_name_contains in migration_name
             )
+            or (
+                self.include_name_contains
+                and self.include_name_contains not in migration_name
+            )
             or (migration_name in self.ignore_name)
+            or (self.include_name and migration_name not in self.include_name)
             or (
                 self.only_applied_migrations
                 and (app_label, migration_name)
@@ -367,12 +401,17 @@ class MigrationLinter(object):
         for operation in migration.operations:
             if isinstance(operation, RunPython):
                 op_errors, op_ignored, op_warnings = self.lint_runpython(operation)
-                if op_errors:
-                    errors += op_errors
-                if op_ignored:
-                    ignored += op_ignored
-                if op_warnings:
-                    warnings += op_warnings
+            elif isinstance(operation, RunSQL):
+                op_errors, op_ignored, op_warnings = self.lint_runsql(operation)
+            else:
+                op_errors, op_ignored, op_warnings = [], [], []
+
+            if op_errors:
+                errors += op_errors
+            if op_ignored:
+                ignored += op_ignored
+            if op_warnings:
+                warnings += op_warnings
 
         return errors, ignored, warnings
 
@@ -382,9 +421,10 @@ class MigrationLinter(object):
         ignored = []
         warning = []
 
+        # Detect warning on missing reverse operation
         if not runpython.reversible:
             issue = {
-                "code": "REVERSIBLE_DATA_MIGRATION",
+                "code": "RUNPYTHON_REVERSIBLE",
                 "msg": "'{}': RunPython data migration is not reversible".format(
                     function_name
                 ),
@@ -394,14 +434,15 @@ class MigrationLinter(object):
             else:
                 warning.append(issue)
 
-        if PY2:
-            args_spec = inspect.getargspec(runpython.code)
-        else:
-            args_spec = inspect.getfullargspec(runpython.code)
-
+        # Detect warning for argument naming convention
+        args_spec = (
+            inspect.getargspec(runpython.code)
+            if PY2
+            else inspect.getfullargspec(runpython.code)
+        )
         if tuple(args_spec.args) != EXPECTED_DATA_MIGRATION_ARGS:
             issue = {
-                "code": "NAMING_CONVENTION_RUNPYTHON_ARGS",
+                "code": "RUNPYTHON_ARGS_NAMING_CONVENTION",
                 "msg": (
                     "'{}': By convention, "
                     "RunPython names the two arguments: apps, schema_editor"
@@ -414,7 +455,7 @@ class MigrationLinter(object):
 
         # Detect wrong model imports
         # Forward
-        issues = self.get_data_migration_model_import_issues(runpython.code)
+        issues = self.get_runpython_model_import_issues(runpython.code)
         for issue in issues:
             if issue["code"] in self.exclude_migration_tests:
                 ignored.append(issue)
@@ -423,35 +464,54 @@ class MigrationLinter(object):
 
         # Backward
         if runpython.reversible:
-            issues = self.get_data_migration_model_import_issues(runpython.reverse_code)
+            issues = self.get_runpython_model_import_issues(runpython.reverse_code)
             for issue in issues:
                 if issue and issue["code"] in self.exclude_migration_tests:
                     ignored.append(issue)
                 else:
                     error.append(issue)
+
+        # Detect warning if model variable name is not the same as model class
+        issues = self.get_runpython_model_variable_naming_issues(runpython.code)
+        for issue in issues:
+            if issue and issue["code"] in self.exclude_migration_tests:
+                ignored.append(issue)
+            else:
+                warning.append(issue)
+
+        if runpython.reversible:
+            issues = self.get_runpython_model_variable_naming_issues(
+                runpython.reverse_code
+            )
+            for issue in issues:
+                if issue and issue["code"] in self.exclude_migration_tests:
+                    ignored.append(issue)
+                else:
+                    warning.append(issue)
+
         return error, ignored, warning
 
     @staticmethod
-    def get_data_migration_model_import_issues(code):
+    def get_runpython_model_import_issues(code):
         model_object_regex = re.compile(r"[^a-zA-Z]?([a-zA-Z0-9]+?)\.objects")
 
         function_name = code.__name__
-        reverse_code_source = inspect.getsource(code)
+        source_code = inspect.getsource(code)
 
-        called_models = model_object_regex.findall(reverse_code_source)
+        called_models = model_object_regex.findall(source_code)
         issues = []
         for model in called_models:
             has_get_model_call = (
                 re.search(
-                    r"get_model\(\s*?.*?,.*?{}.+?\s*?\)".format(model),
-                    reverse_code_source,
+                    r"{}.*= +\w+\.get_model\(".format(model),
+                    source_code,
                 )
                 is not None
             )
             if not has_get_model_call:
                 issues.append(
                     {
-                        "code": "DATA_MIGRATION_MODEL_IMPORT",
+                        "code": "RUNPYTHON_MODEL_IMPORT",
                         "msg": (
                             "'{}': Could not find an 'apps.get_model(\"...\", \"{}\")' "
                             "call. Importing the model directly is incorrect for "
@@ -460,3 +520,118 @@ class MigrationLinter(object):
                     }
                 )
         return issues
+
+    @staticmethod
+    def get_runpython_model_variable_naming_issues(code):
+        model_object_regex = re.compile(r"[^a-zA-Z]?([a-zA-Z0-9]+?)\.objects")
+
+        function_name = code.__name__
+        source_code = inspect.getsource(code)
+
+        called_models = model_object_regex.findall(source_code)
+        issues = []
+        for model in called_models:
+            has_same_model_name = (
+                re.search(
+                    r"{model}.*= +\w+?\.get_model\([^)]+?\.{model}.*?\)".format(
+                        model=model
+                    ),
+                    source_code,
+                    re.MULTILINE | re.DOTALL,
+                )
+                is not None
+                or re.search(
+                    r"{model}.*= +\w+?\.get_model\([^)]+?,[^)]*?{model}.*?\)".format(
+                        model=model
+                    ),
+                    source_code,
+                    re.MULTILINE | re.DOTALL,
+                )
+                is not None
+            )
+            if not has_same_model_name:
+                issues.append(
+                    {
+                        "code": "RUNPYTHON_MODEL_VARIABLE_NAME",
+                        "msg": (
+                            "'{}': Model variable name {} is different from the "
+                            "model class name that was found in the "
+                            "apps.get_model(...) call."
+                        ).format(function_name, model),
+                    }
+                )
+        return issues
+
+    def lint_runsql(self, runsql):
+        error = []
+        ignored = []
+        warning = []
+
+        # Detect warning on missing reverse operation
+        if not runsql.reversible:
+            issue = {
+                "code": "RUNSQL_REVERSIBLE",
+                "msg": "RunSQL data migration is not reversible",
+            }
+            if issue["code"] in self.exclude_migration_tests:
+                ignored.append(issue)
+            else:
+                warning.append(issue)
+
+        # Put the SQL in our SQL analyser
+        if runsql.sql != RunSQL.noop:
+            sql_statements = []
+            if isinstance(runsql.sql, (list, tuple)):
+                for sql in runsql.sql:
+                    params = None
+                    if isinstance(sql, (list, tuple)):
+                        elements = len(sql)
+                        if elements == 2:
+                            sql, params = sql
+                        else:
+                            raise ValueError("Expected a 2-tuple but got %d" % elements)
+                        sql_statements.append(sql % params)
+                    else:
+                        sql_statements.append(sql)
+            else:
+                sql_statements.append(runsql.sql)
+
+            sql_errors, sql_ignored = analyse_sql_statements(
+                sql_statements,
+                settings.DATABASES[self.database]["ENGINE"],
+                self.exclude_migration_tests,
+            )
+            if sql_errors:
+                error += sql_errors
+            if sql_ignored:
+                ignored += sql_ignored
+
+        # And analysse the reverse SQL
+        if runsql.reversible and runsql.reverse_sql != RunSQL.noop:
+            sql_statements = []
+            if isinstance(runsql.reverse_sql, (list, tuple)):
+                for sql in runsql.reverse_sql:
+                    params = None
+                    if isinstance(sql, (list, tuple)):
+                        elements = len(sql)
+                        if elements == 2:
+                            sql, params = sql
+                        else:
+                            raise ValueError("Expected a 2-tuple but got %d" % elements)
+                        sql_statements.append(sql % params)
+                    else:
+                        sql_statements.append(sql)
+            else:
+                sql_statements.append(runsql.reverse_sql)
+
+            sql_errors, sql_ignored = analyse_sql_statements(
+                sql_statements,
+                settings.DATABASES[self.database]["ENGINE"],
+                self.exclude_migration_tests,
+            )
+            if sql_errors:
+                error += sql_errors
+            if sql_ignored:
+                ignored += sql_ignored
+
+        return error, ignored, warning
